@@ -114,6 +114,8 @@
 
 */
 
+#define _XOPEN_SOURCE	500
+
 /*****************************************************************************
                             Include header files
 ******************************************************************************/
@@ -124,6 +126,7 @@
 #include <grp.h>
 #include <ctype.h>
 #include <errno.h>
+#include <sys/time.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -133,6 +136,12 @@
 #include <lcmaps/lcmaps_arguments.h>
 #include <lcmaps/lcmaps_cred_data.h>
 
+#if defined(HAVE_LCMAPS_LCMAPS_PLUGIN_PROTOTYPES_H)
+#   include <lcmaps/lcmaps_plugin_prototypes.h>
+#else
+#   include "lcmaps_plugin_prototypes.h"
+#endif
+
 #include "ldap.h"
 /******************************************************************************
                                 Definitions
@@ -159,11 +168,14 @@
 
 
 #ifndef MAX_LOG_BUFFER_SIZE
-    #define MAX_LOG_BUFFER_SIZE 2048 /*!< Maximum logging buffer size, length of log
-                                          may not exceed this number \internal */
+    #define MAX_LOG_BUFFER_SIZE ((size_t)2048) /*!< Maximum logging buffer size,
+						 length of log may not exceed
+						 this number \internal */
 #endif
 
+#if 0
 #define WHITESPACE_CHARS " \n"
+#endif
 
 /******************************************************************************
                           Module specific prototypes
@@ -171,12 +183,12 @@
 
 static int plugin_run_or_verify(int, lcmaps_argument_t *, int);
 static int log_cred(
-        char * type,
-        char * add_log
+        const char * type,
+        const char * add_log
 );
 static int lcmaps_get_ldap_pw(
         const char * path,
-        char ** ldap_passwd
+        char ** myldap_passwd
 );
 static int lcmaps_set_pgid(
         const char * username,
@@ -232,26 +244,23 @@ Returns:
     0: success
     1: failure
 ******************************************************************************/
-static int log_cred (char * type, char * add_log)
+static int log_cred (const char * type, const char * add_log)
 {
-    char * logstr = "lcmaps_plugin_ldap-log_cred()";
-    char * tmp = NULL;
+    const char * logstr = "lcmaps_plugin_ldap-log_cred()";
+    size_t len;
 
     if (strcmp(type, "UID") == 0)
     {
         if (ldap_cred_log_uid == NULL)
         {
             ldap_cred_log_uid  = (char *) malloc(MAX_LOG_BUFFER_SIZE * sizeof(char));
-
-            strcpy(ldap_cred_log_uid, "uid=");
-            strcat (ldap_cred_log_uid, add_log);
+	    snprintf(ldap_cred_log_uid, MAX_LOG_BUFFER_SIZE, "uid=%s", add_log); 
         }
         else
         {
-            tmp = (char *) malloc(3 * sizeof(char));
-            strcpy (tmp, ",");
-            strcat (ldap_cred_log_uid, tmp);
-            strcat (ldap_cred_log_uid, add_log);
+	    len=strlen(ldap_cred_log_uid);
+	    if (len<MAX_LOG_BUFFER_SIZE)
+		snprintf(ldap_cred_log_uid+len, MAX_LOG_BUFFER_SIZE-len, ",%s", add_log); 
         }
 
         lcmaps_log_debug(5,"%s: ldap_cred_log_uid: %s\n", logstr, ldap_cred_log_uid);
@@ -262,16 +271,13 @@ static int log_cred (char * type, char * add_log)
         if (ldap_cred_log_pgid == NULL)
         {
             ldap_cred_log_pgid  = (char *) malloc(MAX_LOG_BUFFER_SIZE * sizeof(char));
-
-            strcpy(ldap_cred_log_pgid, "pgid=");
-            strcat (ldap_cred_log_pgid, add_log);
+	    snprintf(ldap_cred_log_pgid, MAX_LOG_BUFFER_SIZE, "pgid=%s", add_log);
         }
         else
         {
-            tmp = (char *) malloc(3 * sizeof(char));
-            strcpy (tmp, ",");
-            strcat (ldap_cred_log_pgid, tmp);
-            strcat (ldap_cred_log_pgid, add_log);
+	    len=strlen(ldap_cred_log_pgid);
+	    if (len<MAX_LOG_BUFFER_SIZE)
+		snprintf(ldap_cred_log_pgid+len, MAX_LOG_BUFFER_SIZE-len, ",%s", add_log);
         }
 
         lcmaps_log_debug(5,"%s: ldap_cred_log_pgid: %s\n", logstr, ldap_cred_log_pgid);
@@ -281,41 +287,40 @@ static int log_cred (char * type, char * add_log)
         if (ldap_cred_log_sgid == NULL)
         {
             ldap_cred_log_sgid  = (char *) malloc(MAX_LOG_BUFFER_SIZE * sizeof(char));
-
-            strcpy(ldap_cred_log_sgid, "sgid=");
-            strcat (ldap_cred_log_sgid, add_log);
+	    snprintf(ldap_cred_log_sgid, MAX_LOG_BUFFER_SIZE, "sgid=%s", add_log);
         }
         else
         {
-            tmp = (char *) malloc(3 * sizeof(char));
-            strcpy (tmp, ",");
-            strcat (ldap_cred_log_sgid, tmp);
-            strcat (ldap_cred_log_sgid, add_log);
+	    len=strlen(ldap_cred_log_sgid);
+	    if (len<MAX_LOG_BUFFER_SIZE)
+		snprintf(ldap_cred_log_sgid+len, MAX_LOG_BUFFER_SIZE-len, ",%s", add_log);
         }
 
         lcmaps_log_debug(5,"%s: ldap_cred_log_sgid: %s\n", logstr, ldap_cred_log_sgid);
     }
     else if (strcmp(type, "LOG") == 0)
     {
-        tmp = strdup (":");
-
-
         if (ldap_cred_log == NULL)
-            ldap_cred_log = (char *) malloc (MAX_LOG_BUFFER_SIZE * sizeof(char));
+            ldap_cred_log = (char *) calloc (MAX_LOG_BUFFER_SIZE, sizeof(char));
 
-        if (ldap_cred_log_uid != NULL)
-            strcpy(ldap_cred_log, ldap_cred_log_uid);
+        if (ldap_cred_log_uid != NULL)	{
+	    len=strlen(ldap_cred_log);
+	    if (len<MAX_LOG_BUFFER_SIZE)
+		snprintf(ldap_cred_log+len, MAX_LOG_BUFFER_SIZE-len, "%s", ldap_cred_log_uid);
+	}
 
         if (ldap_cred_log_pgid != NULL)
         {
-            strcat(ldap_cred_log, tmp);
-            strcat(ldap_cred_log, ldap_cred_log_pgid);
+	    len=strlen(ldap_cred_log);
+	    if (len<MAX_LOG_BUFFER_SIZE)
+		snprintf(ldap_cred_log+len, MAX_LOG_BUFFER_SIZE-len, ":%s", ldap_cred_log_pgid);
         }
 
         if (ldap_cred_log_sgid != NULL)
         {
-            strcat(ldap_cred_log, tmp);
-            strcat(ldap_cred_log, ldap_cred_log_sgid);
+	    len=strlen(ldap_cred_log);
+	    if (len<MAX_LOG_BUFFER_SIZE)
+		snprintf(ldap_cred_log+len, MAX_LOG_BUFFER_SIZE-len, ":%s", ldap_cred_log_sgid);
         }
 
         lcmaps_log(LOG_INFO, "%s: %s\n", logstr, ldap_cred_log);
@@ -342,8 +347,10 @@ int plugin_initialize(
         char ** argv
 )
 {
-    char *  logstr = "lcmaps_plugin_ldap_enf-plugin_initialize()";
-    int i, j;
+    const char * logstr = "lcmaps_plugin_ldap_enf-plugin_initialize()";
+    int i;
+    size_t j;
+    long long_val;
 
     lcmaps_log_debug(5,"%s: passed arguments:\n", logstr);
     for (i=0; i < argc; i++)
@@ -356,7 +363,7 @@ int plugin_initialize(
      */
     for (i = 1; i < argc; i++)
     {
-        // setting maxuid parameter from init plugin arguments
+        /*setting maxuid parameter from init plugin arguments */
         if ( (((strcmp(argv[i], "-maxuid") == 0) ||
              (strcmp(argv[i], "-MAXUID") == 0)) &&
              (maxuid == MAX_UNDEFINED))
@@ -370,11 +377,17 @@ int plugin_initialize(
                      if (!isdigit((argv[i + 1])[j]))
                      {
                          lcmaps_log(LOG_ERR,"%s: Error, maybe found some digits, but there is at least one char corrupting this parameter: %s\n", logstr, argv[i + 1]);
-                         maxuid = -1;
                          goto fail_ldap;
                      }
                  }
-                 maxuid = atoi(argv[i + 1]);
+		 errno=0;
+		 long_val=strtol(argv[i + 1], NULL, 10);
+		 if (errno!=0 || long_val<0)	{
+		     lcmaps_log(LOG_ERR,"%s: cannot convert %s to maxuid.\n",
+			     logstr, argv[i + 1]);
+		     goto fail_ldap;
+		 }
+		 maxuid = (int)long_val;
             }
             else
             {
@@ -384,7 +397,7 @@ int plugin_initialize(
             i++;
         }
 
-        // setting maxpgid parameter from init plugin arguments
+        /*setting maxpgid parameter from init plugin arguments */
         else if ( (((strcmp(argv[i], "-maxpgid") == 0) ||
              (strcmp(argv[i], "-MAXPGID") == 0)) &&
              (maxpgid == MAX_UNDEFINED))
@@ -398,11 +411,17 @@ int plugin_initialize(
                      if (!isdigit((argv[i + 1])[j]))
                      {
                          lcmaps_log(LOG_ERR,"%s: Error, maybe found some digits, but there is at least one char corrupting this parameter: %s\n", logstr, argv[i + 1]);
-                         maxpgid = -1;
                          goto fail_ldap;
                      }
                  }
-                 maxpgid = atoi(argv[i + 1]);
+		 errno=0;
+		 long_val=strtol(argv[i + 1], NULL, 10);
+		 if (errno!=0 || long_val<0)	{
+		     lcmaps_log(LOG_ERR,"%s: cannot convert %s to maxpgid.\n",
+			     logstr, argv[i + 1]);
+		     goto fail_ldap;
+		 }
+		 maxpgid = (int)long_val;
             }
             else
             {
@@ -412,7 +431,7 @@ int plugin_initialize(
             i++;
         }
 
-        // setting maxsgid parameter from init plugin arguments
+        /*setting maxsgid parameter from init plugin arguments */
         else if  ( (((strcmp(argv[i], "-maxsgid") == 0) ||
              (strcmp(argv[i], "-MAXSGID") == 0)) &&
              (maxsgid == MAX_UNDEFINED))
@@ -426,11 +445,17 @@ int plugin_initialize(
                      if (!isdigit((argv[i + 1])[j]))
                      {
                          lcmaps_log(LOG_ERR,"%s: Error, maybe found some digits, but there is atleast one char corrupting this parameter: %s\n", logstr, argv[i + 1]);
-                         maxsgid = -1;
                          goto fail_ldap;
                      }
                  }
-                 maxsgid = atoi(argv[i + 1]);
+		 errno=0;
+		 long_val=strtol(argv[i + 1], NULL, 10);
+		 if (errno!=0 || long_val<0)	{
+		     lcmaps_log(LOG_ERR,"%s: cannot convert %s to maxsgid.\n",
+			     logstr, argv[i + 1]);
+		     goto fail_ldap;
+		 }
+		 maxsgid = (int)long_val;
             }
             else
             {
@@ -456,7 +481,14 @@ int plugin_initialize(
                          goto fail_ldap;
                      }
                  }
-                 port = atoi(argv[i + 1]);
+		 errno=0;
+		 long_val=strtol(argv[i + 1], NULL, 10);
+		 if (errno!=0 || long_val<0)	{
+		     lcmaps_log(LOG_ERR,"%s: cannot convert %s to port.\n",
+			     logstr, argv[i + 1]);
+		     goto fail_ldap;
+		 }
+		 port = (int)long_val;
             }
             else
             {
@@ -562,7 +594,7 @@ int plugin_initialize(
             }
             i++;
         }
-        // setting timeout value parameter (in seconds) from init plugin arguments
+        /*setting timeout value parameter (in seconds) from init plugin arguments */
         else if ( (strcmp(argv[i], "-timeout") == 0)
              && (i + 1 < argc) )
         {
@@ -578,7 +610,14 @@ int plugin_initialize(
                          goto fail_ldap;
                      }
                  }
-                 timeout.tv_sec = (time_t) atoi(argv[i + 1]);
+		 errno=0;
+		 long_val=strtol(argv[i + 1], NULL, 10);
+		 if (errno!=0 || long_val<0)	{
+		     lcmaps_log(LOG_ERR,"%s: cannot convert %s to timeout.tv_sec.\n",
+			     logstr, argv[i + 1]);
+		     goto fail_ldap;
+		 }
+		 timeout.tv_sec = (time_t) long_val;
                  timeout.tv_usec = (suseconds_t) 0;
             }
             else
@@ -645,8 +684,8 @@ int plugin_initialize(
     lcmaps_log_debug(2,"%s: Summary init ldap_pw      : %s\n", logstr, ldap_pw);
     lcmaps_log_debug(2,"%s: Summary init sb_groups    : %s\n", logstr, sb_groups);
     lcmaps_log_debug(2,"%s: Summary init sb_user      : %s\n", logstr, sb_user);
-    lcmaps_log_debug(2,"%s: Summary init timeout      : %d seconds and %d microseconds\n", logstr,
-    (int) timeout.tv_sec, (int) timeout.tv_usec);
+    lcmaps_log_debug(2,"%s: Summary init timeout      : %ld seconds and %ld microseconds\n", logstr,
+    (long) timeout.tv_sec, (long) timeout.tv_usec);
 
     return LCMAPS_MOD_SUCCESS;
 
@@ -669,7 +708,7 @@ int plugin_introspect(
         lcmaps_argument_t ** argv
 )
 {
-    char *                   logstr    = "lcmaps_plugin_ldap_enf-plugin_introspect()";
+    const char *             logstr    = "lcmaps_plugin_ldap_enf-plugin_introspect()";
     static lcmaps_argument_t argList[] = {
         {NULL           ,       NULL            , -1,   NULL}
     };
@@ -678,7 +717,7 @@ int plugin_introspect(
 
     *argv = argList;
     *argc = lcmaps_cntArgs(argList);
-    lcmaps_log_debug(5,"%s: address first argument: 0x%x\n", logstr, argList);
+    lcmaps_log_debug(5,"%s: address first argument: %p\n", logstr, (void*)argList);
 
     return LCMAPS_MOD_SUCCESS;
 }
@@ -734,7 +773,7 @@ static int plugin_run_or_verify(
         int lcmaps_mode
 )
 {
-    char *           logstr = "lcmaps_plugin_ldap_enf-plugin_run()";
+    const char     * logstr = "lcmaps_plugin_ldap_enf-plugin_run()";
     struct passwd  * user_info   = NULL;
     int              i;
     gid_t          * list        = NULL;
@@ -769,9 +808,9 @@ static int plugin_run_or_verify(
     }
     lcmaps_log_debug(5,"%s\n", logstr);
 
-    uid    = getCredentialData(UID,     &cntUid);
-    priGid = getCredentialData(PRI_GID, &cntPriGid);
-    secGid = getCredentialData(SEC_GID, &cntSecGid);
+    uid    = (uid_t *)getCredentialData(UID,     &cntUid);
+    priGid = (gid_t *)getCredentialData(PRI_GID, &cntPriGid);
+    secGid = (gid_t *)getCredentialData(SEC_GID, &cntSecGid);
     lcmaps_log_debug(2, "%s: number of uids: %d, priGids: %d, secGids: %d\n", logstr, cntUid, cntPriGid, cntSecGid);
 
 
@@ -809,7 +848,7 @@ static int plugin_run_or_verify(
     /* You must be ROOT */
     if (getuid() != 0)
     {
-        lcmaps_log(LOG_ERR, "%s: You are not ROOT!  -> %d\n", logstr, getuid());
+        lcmaps_log(LOG_ERR, "%s: You are not ROOT!  -> %u\n", logstr, (unsigned)getuid());
         goto fail_ldap;
     }
 
@@ -825,7 +864,7 @@ static int plugin_run_or_verify(
     /* Get a handle to an LDAP connection. */
     if ( (ld_handle = ldap_init( hostname, port)) == NULL )
     {
-      lcmaps_log(LOG_ERR, "%s: no handle to LDAP server on %s using port %s -> %s\n", logstr, hostname, port, strerror(errno));
+      lcmaps_log(LOG_ERR, "%s: no handle to LDAP server on %s using port %d -> %s\n", logstr, hostname, port, strerror(errno));
       goto fail_ldap;
     }
     lcmaps_log_debug(3, "%s: handle to LDAP server successfull\n", logstr);
@@ -873,7 +912,7 @@ static int plugin_run_or_verify(
       }
       else
       {
-        lcmaps_log(LOG_ERR,"%s: Error getting handle to ldap server  -> %s (rc = %d)\n", logstr, rc);
+        lcmaps_log(LOG_ERR,"%s: Error getting handle to ldap server  -> (rc = %d)\n", logstr, rc);
         goto fail_ldap;
       }
       lcmaps_log_debug(5,"%s: freeing ldap msg\n", logstr);
@@ -890,11 +929,11 @@ static int plugin_run_or_verify(
         lcmaps_log(LOG_ERR,"%s: Error, found no userid in credential data repository (failure)\n", logstr);
         goto fail_ldap;
     }
-    lcmaps_log_debug(3, "%s: uid = %d\n", logstr, (int) *uid );
+    lcmaps_log_debug(3, "%s: uid = %u\n", logstr, (unsigned) *uid );
     user_info = getpwuid( *uid);
     if (user_info == NULL)
     {
-        lcmaps_log(LOG_ERR, "%s: Error, no pw_name associated with uid: %d\n", logstr, uid);
+        lcmaps_log(LOG_ERR, "%s: Error, no pw_name associated with uid: %u\n", logstr, (unsigned)(*uid));
         goto fail_ldap;
     }
     lcmaps_log_debug(3, "%s: uidname = %s\n", logstr, user_info->pw_name );
@@ -907,31 +946,37 @@ static int plugin_run_or_verify(
         {
             lcmaps_log_debug(3, "%s: priGidName = %s\n", logstr, grgid->gr_name);
             /* set the pgid */
-             if ( lcmaps_set_pgid(user_info->pw_name, grgid->gr_name, (gid_t)(int) grgid->gr_gid, ld_handle, sb_user) != 0 )
+             if ( lcmaps_set_pgid(user_info->pw_name, grgid->gr_name, grgid->gr_gid, ld_handle, sb_user) != 0 )
              {
-                 lcmaps_log(LOG_ERR, "%s: lcmaps_set_pgid failed for user=%s, pri group=%s, pri gid=%d\n", logstr,
-                            user_info->pw_name, grgid->gr_name, (int) grgid->gr_gid);
+                 lcmaps_log(LOG_ERR, "%s: lcmaps_set_pgid failed for user=%s, pri group=%s, pri gid=%u\n", logstr,
+                            user_info->pw_name, grgid->gr_name, (unsigned)(grgid->gr_gid));
                  goto fail_ldap;
              }
-             // Add to log if succesfull
+             /*Add to log if succesfull */
              else
              {
-                  // Log User ID with it's Username
+                  /*Log User ID with it's Username */
                   tmplog = (char *) malloc (MAX_LOG_BUFFER_SIZE * sizeof(char));
-                  snprintf(tmplog, MAX_LOG_BUFFER_SIZE-1, "%d(%s)", user_info->pw_uid, user_info->pw_name);
+                  snprintf(tmplog, MAX_LOG_BUFFER_SIZE-1, "%u(%s)", (unsigned)(user_info->pw_uid), user_info->pw_name);
                   log_cred("UID", tmplog);
-                  if (tmplog) free(tmplog); tmplog = NULL;
+                  if (tmplog)	{
+		      free(tmplog);
+		      tmplog = NULL;
+		  }
 
-                  // Primary Group ID and Group Name
+                  /*Primary Group ID and Group Name */
                   tmplog = (char *) malloc (MAX_LOG_BUFFER_SIZE * sizeof(char));
-                  snprintf(tmplog, MAX_LOG_BUFFER_SIZE-1, "%d(%s)", grgid->gr_gid, grgid->gr_name);
+                  snprintf(tmplog, MAX_LOG_BUFFER_SIZE-1, "%u(%s)", (unsigned)(grgid->gr_gid), grgid->gr_name);
                   log_cred("PGID", tmplog);
-                  if (tmplog) free(tmplog); tmplog = NULL;
+                  if (tmplog)	{
+		      free(tmplog);
+		      tmplog = NULL;
+		  }
              }
         }
         else
         {
-            lcmaps_log(LOG_ERR, "%s: Error, No grgid name associated with primary gid number: %d\n", logstr, (int) priGid[0]);
+            lcmaps_log(LOG_ERR, "%s: Error, No grgid name associated with primary gid number: %u\n", logstr, (unsigned) priGid[0]);
             goto fail_ldap;
         }
     }
@@ -940,7 +985,7 @@ static int plugin_run_or_verify(
     i=0;
     while (i < cntSecGid)
     {
-        lcmaps_log_debug(3, "%s: secGid = %d\n", logstr, (int) secGid[i]);
+        lcmaps_log_debug(3, "%s: secGid = %u\n", logstr, (unsigned) secGid[i]);
         grgid = getgrgid((gid_t) secGid[i]);
         if (grgid != NULL)
         {
@@ -950,28 +995,31 @@ static int plugin_run_or_verify(
             {
                 if (require_all_groups == 1)
                 {
-                    lcmaps_log(LOG_ERR, "%s: failure for user=%s, group=%s, gid=%d\n", logstr,
-                           user_info->pw_name, grgid->gr_name, (int) grgid->gr_gid);
+                    lcmaps_log(LOG_ERR, "%s: failure for user=%s, group=%s, gid=%u\n", logstr,
+                           user_info->pw_name, grgid->gr_name, (unsigned) grgid->gr_gid);
                     goto fail_ldap;
                 }
                 else
                 {
-                    lcmaps_log_debug(1, "%s: failure for user=%s, group=%s, gid=%d\n", logstr,
-                           user_info->pw_name, grgid->gr_name, (int) grgid->gr_gid);
+                    lcmaps_log_debug(1, "%s: failure for user=%s, group=%s, gid=%u\n", logstr,
+                           user_info->pw_name, grgid->gr_name, (unsigned) grgid->gr_gid);
                 }
             }
             else
             {
-                // Add the secondary group IDs & names
+                /*Add the secondary group IDs & names */
                 tmplog = (char *) malloc (MAX_LOG_BUFFER_SIZE * sizeof(char));
-                snprintf(tmplog, MAX_LOG_BUFFER_SIZE-1, "%d(%s)", grgid->gr_gid, grgid->gr_name);
+                snprintf(tmplog, MAX_LOG_BUFFER_SIZE-1, "%u(%s)", (unsigned)(grgid->gr_gid), grgid->gr_name);
                 log_cred("SGID", tmplog);
-                if (tmplog) free(tmplog); tmplog = NULL;
+                if (tmplog) {
+		    free(tmplog);
+		    tmplog = NULL;
+		}
             }
         }
         else
         {
-            lcmaps_log(LOG_INFO, "%s: Error, No grgid name associated with gid number: %d\n", logstr, (int) secGid[i]);
+            lcmaps_log(LOG_INFO, "%s: Error, No grgid name associated with gid number: %u\n", logstr, (unsigned) secGid[i]);
         }
         i++;
     }
@@ -982,20 +1030,22 @@ static int plugin_run_or_verify(
 
 
     /* succes */
- success_ldap:
-    if (ld_handle) ldap_unbind_s(ld_handle);
+/* success_ldap:*/
+    if (ld_handle)
+	ldap_unbind_s(ld_handle);
     lcmaps_log(LOG_INFO,"%s: ldap_enf plugin succeeded\n", logstr);
     return LCMAPS_MOD_SUCCESS;
 
  fail_ldap:
-    // added:
+    /*added: */
     if (ldap_bind_msg)
     {
       ldap_msgfree(ldap_bind_msg);
       ldap_bind_msg = NULL;
     }
-    if (ld_handle) ldap_unbind_s(ld_handle);
-    if (list) free(list);
+    if (ld_handle)
+	ldap_unbind_s(ld_handle);
+    free(list);
     lcmaps_log(LOG_INFO,"%s: ldap_enf plugin failed\n", logstr);
     return LCMAPS_MOD_FAIL;
 }
@@ -1010,9 +1060,9 @@ Returns:
     LCMAPS_MOD_SUCCESS : succes
     LCMAPS_MOD_FAIL    : failure
 ******************************************************************************/
-int plugin_terminate()
+int plugin_terminate(void)
 {
-    char *              logstr = "lcmaps_plugin_ldap_enf-plugin_terminate()";
+    const char * logstr = "lcmaps_plugin_ldap_enf-plugin_terminate()";
 
     lcmaps_log_debug(4,"%s: terminating\n", logstr);
 
@@ -1101,22 +1151,23 @@ static int lcmaps_add_username_to_ldapgroup(
         const char * searchBase
     )
 {
-    char *         logstr = "lcmaps_plugin_ldap_enf-lcmaps_add_username_to_ldapgroup()";
+    const char *   logstr = "lcmaps_plugin_ldap_enf-lcmaps_add_username_to_ldapgroup()";
     int            i = 0;
-    int            filtersize;
-    char         * filterprefix = "(&(objectClass=posixGroup)(gidNumber=";
+    size_t         filtersize;
+    const char   * filterprefix = "(&(objectClass=posixGroup)(gidNumber=";
     char         * filter = NULL;
-    char         * filtersuffix = "))";
-    char         * memberUidfilter = ")(memberUid=";
+    const char   * filtersuffix = "))";
+    const char   * memberUidfilter = ")(memberUid=";
     char         * memberfilter;
     LDAPMessage  * searchResult, *entry;
     char         * attribute, **values;
-    struct timeval timeOut = {10,0};
+    struct timeval timeOut = {10L,0L};
     char           groupstr[10];
     char         * dn = NULL;
     BerElement   * ber;
-    int            rc, entryCount, dnsize, membersize;
-    char         * dnprefix = "cn=";
+    int            rc, entryCount;
+    size_t         dnsize, membersize;
+    const char   * dnprefix = "cn=";
     LDAPMod        modGID;
     LDAPMod      * modify[2];
     char         * modifyDN;
@@ -1129,32 +1180,32 @@ static int lcmaps_add_username_to_ldapgroup(
     }
     if (groupnumber > MAXGROUPNUM)
     {
-        lcmaps_log(LOG_ERR,"%s: groupid (%d) too large, max = %d\n", logstr,
-        groupnumber,MAXGROUPNUM);
+        lcmaps_log(LOG_ERR,"%s: groupid (%u) too large, max = %d\n", logstr,
+	    (unsigned)groupnumber,MAXGROUPNUM);
         return 1;
     }
 
-    snprintf(groupstr,(size_t)10,"%d",groupnumber);
+    snprintf(groupstr,(size_t)10,"%u",(unsigned)groupnumber);
     filtersize = strlen(filterprefix)+strlen(groupstr)+strlen(filtersuffix)+1;
     filter = (char *) malloc(filtersize*sizeof(char));
 
     snprintf(filter,(size_t)filtersize,"%s%s%s",filterprefix, groupstr, filtersuffix);
 
     lcmaps_log_debug(3,"%s: groupstr = %s\n", logstr,groupstr);
-    lcmaps_log_debug(3,"%s: filter: %s, filtersize = %d\n", logstr,filter, filtersize);
-    lcmaps_log_debug(3,"%s: username: %s, groupnumber = %d, ld_handle = %p, searchBase = %s\n", logstr,
-                     username, groupnumber, ld_handle, searchBase);
+    lcmaps_log_debug(3,"%s: filter: %s, filtersize = %lu\n", logstr,filter, (long unsigned)filtersize);
+    lcmaps_log_debug(3,"%s: username: %s, groupnumber = %u, ld_handle = %p, searchBase = %s\n", logstr,
+                     username, (unsigned)groupnumber, (void*)ld_handle, searchBase);
 
     membersize = strlen(filterprefix) + strlen(groupstr) + strlen(memberUidfilter) + strlen(username) + strlen(filtersuffix) +1;
     memberfilter = (char *) malloc(membersize*sizeof(char));
     snprintf(memberfilter,(size_t)memberfilter,"%s%s%s%s%s",filterprefix, groupstr,memberUidfilter,username ,filtersuffix);
-    lcmaps_log_debug(3,"%s: memberefilter: %s, membersize = %d\n", logstr,memberfilter, membersize);
+    lcmaps_log_debug(3,"%s: memberefilter: %s, membersize = %lu\n", logstr,memberfilter, (long unsigned)membersize);
 
-    dnsize = strlen(dnprefix) + strlen(groupname)  + strlen(searchBase) +2;  // '/0' and ','
+    dnsize = strlen(dnprefix) + strlen(groupname)  + strlen(searchBase) +2;  /*'/0' and ',' */
     modifyDN = (char *) malloc(dnsize*sizeof(char));
     snprintf(modifyDN,(size_t)dnsize,"%s%s,%s",dnprefix,groupname,searchBase );
     lcmaps_log_debug(3,"%s: modifyDN = %s\n", logstr,modifyDN);
-    //modifyDN = "cn=tbadmin,ou=LocalGroups,dc=foobar,dc=ough";
+    /*modifyDN = "cn=tbadmin,ou=LocalGroups,dc=foobar,dc=ough"; */
 
 
     modGID.mod_op     = LDAP_MOD_ADD;
@@ -1214,7 +1265,7 @@ static int lcmaps_add_username_to_ldapgroup(
 
     if ( rc != LDAP_SUCCESS )
     {
-        lcmaps_log(LOG_ERR,"%s: ldap_modify_ext_s failed for group %s(%d): %s\n", logstr, groupname, groupnumber, ldap_err2string( rc ));
+        lcmaps_log(LOG_ERR,"%s: ldap_modify_ext_s failed for group %s(%u): %s\n", logstr, groupname, (unsigned)groupnumber, ldap_err2string( rc ));
         free(GIDValue[0]);
         free(filter);
         free(modifyDN);
@@ -1231,7 +1282,7 @@ static int lcmaps_add_username_to_ldapgroup(
                     searchBase,            /* container to search */
                     LDAP_SCOPE_SUBTREE,
                     filter,
-//                    &attrstr,              /* return all attributes */
+/*                   &attrstr,              \* return all attributes *\ */
                     NULL,                  /* return all attributes */
                     0,                     /* return attributes and values */
                     NULL,                  /* server controls */
@@ -1341,19 +1392,20 @@ static int lcmaps_set_pgid(
         const char * searchBase
     )
 {
-    char *         logstr = "lcmaps_plugin_ldap_enf-lcmaps_set_pgid()";
-    int            rc, dnsize;
+    const char *   logstr = "lcmaps_plugin_ldap_enf-lcmaps_set_pgid()";
+    int            rc;
+    size_t         dnsize;
     char           prigroupstr[10];
     LDAPMod        modGID;
     LDAPMod      * modify[2];
     char         * modifyDN = NULL;
     char         * GIDValue[2];
-    char         * dnprefix = "uid=";
+    const char   * dnprefix = "uid=";
 
 
     /* print incoming vars: */
-    lcmaps_log_debug(5, "%s: parameters: user=%s, pri group=%s, pri gid=%d\n", logstr,
-               username, pgroupname, (int) pgroupnumber);
+    lcmaps_log_debug(5, "%s: parameters: user=%s, pri group=%s, pri gid=%u\n", logstr,
+               username, pgroupname, (unsigned) pgroupnumber);
 
     if (! ld_handle)
     {
@@ -1362,17 +1414,17 @@ static int lcmaps_set_pgid(
     }
     if (pgroupnumber > MAXGROUPNUM)
     {
-        lcmaps_log(LOG_ERR,"%s: pgroupid (%d) too large, max = %d\n", logstr,
-        pgroupnumber,MAXGROUPNUM);
+        lcmaps_log(LOG_ERR,"%s: pgroupid (%u) too large, max = %d\n", logstr,
+        (unsigned)pgroupnumber,MAXGROUPNUM);
         return 1;
     }
 
-    snprintf(prigroupstr,(size_t)10,"%d", pgroupnumber);
+    snprintf(prigroupstr,(size_t)10,"%u", (unsigned)pgroupnumber);
 
     /* modifyDN    the object to modify */
-    dnsize = strlen(dnprefix) + strlen(username)  + strlen(searchBase) +3;  // '/0' and ','
+    dnsize = strlen(dnprefix) + strlen(username)  + strlen(searchBase) +3;  /*'/0' and ',' */
     modifyDN = (char *) malloc(dnsize*sizeof(char));
-    snprintf(modifyDN,(size_t)dnsize,"%s%s, %s",dnprefix,username,searchBase );
+    snprintf(modifyDN,dnsize,"%s%s, %s",dnprefix,username,searchBase );
     lcmaps_log_debug(3,"%s: modifyDN = %s\n", logstr,modifyDN);
 
 
@@ -1435,7 +1487,7 @@ Returns:
 
 
     \param path     the path to the ldap_pw file containing the password.
-    \param ldap_passwd variable to set the password in
+    \param myldap_passwd variable to set the password in
 
     \retval 0 success
     \retval 1 other failure
@@ -1443,10 +1495,10 @@ Returns:
 
 static int lcmaps_get_ldap_pw(
         const char * path,
-        char       ** ldap_passwd
+        char       ** myldap_passwd
     )
 {
-    char *        logstr = "lcmaps_plugin_ldap_enf-lcmaps_get_ldap_pw()";
+    const char *  logstr = "lcmaps_plugin_ldap_enf-lcmaps_get_ldap_pw()";
     FILE        * fp = NULL;
     struct stat   buf;
     int           c;
@@ -1492,14 +1544,15 @@ static int lcmaps_get_ldap_pw(
     /* I thought tabs and spaces were not allowed in the passwd, but they are, so
      * restore original line
      */
-//    while ((c = getc(fp)) != EOF && (strchr(WHITESPACE_CHARS, c) == NULL)) count++;
+/*    while ((c = getc(fp)) != EOF && (strchr(WHITESPACE_CHARS, c) == NULL)) count++;*/
     while ((c = getc(fp)) != EOF && (c != '\n')) count++;
-    temppwd = (char *) malloc(count * sizeof(char) + 2);
+    temppwd = (char *) malloc((size_t)count * sizeof(char) + 2);
 
     if ((fseek(fp, 0L, SEEK_SET)) != 0)
     {
         lcmaps_log(LOG_ERR, "%s: unable to reset the fp\n", logstr);
         fclose(fp);
+	free(temppwd);
         return 1;
     }
 
@@ -1507,16 +1560,14 @@ static int lcmaps_get_ldap_pw(
     {
         lcmaps_log(LOG_ERR, "%s: unable to read the password: fgets failed\n", logstr);
         fclose(fp);
+	free(temppwd);
         return 1;
     }
 
     fclose(fp);
     lcmaps_log_debug(3,"%s: password: %s\n", logstr, temppwd);
-//    *ldap_passwd = strdup(" s ts   ");
-    *ldap_passwd = strdup(temppwd);
-
-    /* free mem */
-    if (temppwd) free(temppwd);
+/*    *ldap_passwd = strdup(" s ts   ");*/
+    *myldap_passwd = temppwd;
 
     return 0;
 }
