@@ -140,6 +140,20 @@ the privileges of the process from root to that of a local user.
     -# plugin_introspect()
 */
 
+#define _XOPEN_SOURCE	500
+
+#ifdef __sun
+/* For setgroups on Solaris need either no POSIX or define __EXTENSIONS__ */
+# define __EXTENSIONS__
+#elif defined(__APPLE__)
+/* For setgroups on MacOS need either no POSIX or define _DARWIN_C_SOURCE */
+# define _DARWIN_C_SOURCE
+#else
+/* Need BSD_SOURCE for setgroups on Linux */
+# define _BSD_SOURCE
+#endif
+
+
 /*****************************************************************************
                             Include header files
 ******************************************************************************/
@@ -156,7 +170,15 @@ the privileges of the process from root to that of a local user.
 
 #include <lcmaps/lcmaps_modules.h>
 #include <lcmaps/lcmaps_arguments.h>
+
+#if defined(HAVE_LCMAPS_LCMAPS_PLUGIN_PROTOTYPES_H)
+#   include <lcmaps/lcmaps_plugin_prototypes.h>
+#else
+#   include "lcmaps_plugin_prototypes.h"
+#endif
+
 #include <lcmaps/lcmaps_cred_data.h>
+
 /******************************************************************************
                                 Definitions
 ******************************************************************************/
@@ -178,10 +200,6 @@ the privileges of the process from root to that of a local user.
  * a self made define in the code or that it was OS specific.
  */
 
-#ifndef MAX_LOG_BUFFER_SIZE
-    #define MAX_LOG_BUFFER_SIZE 500
-#endif
-
 /******************************************************************************
                           Module specific prototypes
 ******************************************************************************/
@@ -209,7 +227,7 @@ char * cgul_process_identity_oneline(void)
     char * line_buf = NULL;
     struct passwd * p = NULL;
     struct group  * g = NULL;
-    int ngroups = 0;
+    int ngroups;
     gid_t * list = NULL;
 
     uid_t uid  = getuid();
@@ -219,35 +237,35 @@ char * cgul_process_identity_oneline(void)
 
 
     /* Allocate write buffer, must be free'd */
-    if (!(line_buf = calloc (sizeof (char), 1024 + 1)))
+    if (!(line_buf = calloc (sizeof (char), (size_t)(1024 + 1))))
         return NULL;
 
     /* Real User ID */
-    if (p = getpwuid (uid))
-        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), "uid=%d(%s)", uid, p -> pw_name);
+    if ((p = getpwuid (uid)))
+        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), "uid=%u(%s)", (unsigned)uid, p -> pw_name);
     else
-        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), "uid=%d", uid);
+        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), "uid=%u", (unsigned)uid);
 
 
     /* Effective User ID */
-    if (p = getpwuid (euid))
-        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",euid=%d(%s)", euid, p -> pw_name);
+    if ((p = getpwuid (euid)))
+        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",euid=%u(%s)", (unsigned)euid, p -> pw_name);
     else
-        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",euid=%d", euid);
+        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",euid=%u", (unsigned)euid);
 
 
     /* Real Group ID */
-    if (g = getgrgid (gid))
-        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",gid=%d(%s)", gid, g -> gr_name);
+    if ((g = getgrgid (gid)))
+        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",gid=%u(%s)", (unsigned)gid, g -> gr_name);
     else
-        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",gid=%d", gid);
+        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",gid=%u", (unsigned)gid);
 
 
     /* Effective Group ID */
-    if (g = getgrgid (egid))
-        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",egid=%d(%s)", egid, g -> gr_name);
+    if ((g = getgrgid (egid)))
+        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",egid=%u(%s)", (unsigned)egid, g -> gr_name);
     else
-        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",egid=%d", egid);
+        snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",egid=%u", (unsigned)egid);
 
 
     /* Get the number of secondary group IDs for root */
@@ -258,7 +276,7 @@ char * cgul_process_identity_oneline(void)
         errno = stored_errno;
         return NULL;
     }
-    list = (gid_t *) malloc(ngroups * sizeof(gid_t));
+    list = (gid_t *) malloc((size_t)ngroups * sizeof(gid_t));
 
     /* Get the secondary group IDs for root */
     if (getgroups(ngroups, list) < 0)
@@ -273,15 +291,14 @@ char * cgul_process_identity_oneline(void)
     for (t = 0; t < ngroups; t++)
     {
         /* Secondary Group ID */
-        if (g = getgrgid (list[t]))
-            snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",sgid=%d(%s)", list[t], g -> gr_name);
+        if ((g = getgrgid (list[t])))
+            snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",sgid=%u(%s)", (unsigned)(list[t]), g -> gr_name);
         else
-            snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",sgid=%d", list[t]);
+            snprintf (&line_buf[strlen(line_buf)], 1024 - strlen(line_buf), ",sgid=%u", (unsigned)(list[t]));
     }
 
     /* Memory liberation */
-    if (list) free(list);
-    list=(gid_t *) NULL;
+    free(list);
 
     return line_buf;
 }
@@ -305,8 +322,10 @@ int plugin_initialize(
         char ** argv
 )
 {
-    int i, j;
-    char * logstr = "lcmaps_plugin_posix_enf-plugin_initialize()";
+    int i;
+    size_t j;
+    long value;
+    const char * logstr = "lcmaps_plugin_posix_enf-plugin_initialize()";
 
 
     lcmaps_log_debug(5,"%s: passed arguments:\n", logstr);
@@ -334,11 +353,17 @@ int plugin_initialize(
                      if (!isdigit((argv[i + 1])[j]))
                      {
                          lcmaps_log(LOG_ERR,"%s: Error: maybe found some digits, but there is at least one char corrupting this parameter: %s\n", logstr, argv[i + 1]);
-                         maxuid = -1;
                          goto fail_posix;
                      }
                  }
-                 maxuid = atoi(argv[i + 1]);
+		 errno = 0;
+		 value=strtol(argv[i + 1], NULL, 10);
+		 if (errno!=0 || value<0)	{
+		     lcmaps_log(LOG_ERR,"%s: Error: cannot convert %s to maxuid.\n",
+			     logstr,argv[i + 1]);
+		     goto fail_posix;
+		 }
+                 maxuid = (int)value;
             }
             else
             {
@@ -366,7 +391,14 @@ int plugin_initialize(
                          goto fail_posix;
                      }
                  }
-                 maxpgid = atoi(argv[i + 1]);
+		 errno = 0;
+		 value=strtol(argv[i + 1], NULL, 10);
+		 if (errno!=0 || value<0)	{
+		     lcmaps_log(LOG_ERR,"%s: Error: cannot convert %s to maxpgid.\n",
+			     logstr,argv[i + 1]);
+		     goto fail_posix;
+		 }
+                 maxpgid = (int)value;
             }
             else
             {
@@ -394,7 +426,14 @@ int plugin_initialize(
                          goto fail_posix;
                      }
                  }
-                 maxsgid = atoi(argv[i + 1]);
+		 errno = 0;
+		 value=strtol(argv[i + 1], NULL, 10);
+		 if (errno!=0 || value<0)	{
+		     lcmaps_log(LOG_ERR,"%s: Error: cannot convert %s to maxsgid.\n",
+			     logstr,argv[i + 1]);
+		     goto fail_posix;
+		 }
+                 maxsgid = (int)value; 
             }
             else
             {
@@ -494,7 +533,7 @@ int plugin_introspect(
         lcmaps_argument_t ** argv
 )
 {
-    char * logstr = "lcmaps_plugin_posix_enf-plugin_introspect()";  
+    const char * logstr = "lcmaps_plugin_posix_enf-plugin_introspect()";  
 
     static lcmaps_argument_t argList[] = {
         {NULL           ,       NULL            , -1,   NULL}
@@ -504,7 +543,7 @@ int plugin_introspect(
 
     *argv = argList;
     *argc = lcmaps_cntArgs(argList);
-    lcmaps_log_debug(5,"%s: address first argument: 0x%x\n", logstr, argList);
+    lcmaps_log_debug(5,"%s: address first argument: %p\n", logstr, (void*)argList);
 
     return LCMAPS_MOD_SUCCESS;
 }
@@ -560,19 +599,18 @@ static int plugin_run_or_verify(
         int lcmaps_mode
 )
 {
-    char * logstr = "lcmaps_plugin_posix_enf-plugin_run()";  
+    const char * logstr = "lcmaps_plugin_posix_enf-plugin_run()";  
     char * buffer = NULL;
-    int                 i, t; 
+    int                 i;
     gid_t *             list        = NULL;
-    int                 ngroups     = 0;
  
-    char  *             dn          = NULL;
     uid_t *             uid         = NULL;
     int                 cntUid;
     gid_t *             priGid      = NULL;
     int                 cntPriGid;
     gid_t *             secGid      = NULL;
     int                 cntSecGid;
+    int			rc_sg;
 
     /*
      * The beginning
@@ -616,7 +654,7 @@ static int plugin_run_or_verify(
         /* lcmaps_log_debug(2,"%s: max number of secondary gid's: %d\n", logstr, maxsgid); */
         if (cntSecGid > maxsgid)
         {
-            lcmaps_log(LOG_ERR, "%s: Error: The set amount of secondary gid's gathered exceeds the maximum of %d secunadary gid's by %d\n", logstr, maxsgid, (cntSecGid - maxsgid));
+            lcmaps_log(LOG_ERR, "%s: Error: The set amount of secondary gid's gathered exceeds the maximum of %d secundary gid's by %d\n", logstr, maxsgid, (cntSecGid - maxsgid));
             goto fail_posix;
         }
     }
@@ -651,7 +689,10 @@ static int plugin_run_or_verify(
     if (cntPriGid > 0)
     {
         if (set_only_egid)
-            if (setregid(-1, priGid[0]) != 0)
+	    /* Note: setregid() -- both POSIX and Linux -- accepts -1 for the
+	     * first or second argument, even though a gid_t is an unsigned type
+	     * on Linux systems. */
+            if (setregid((gid_t)-1, priGid[0]) != 0)
             {
                 lcmaps_log(LOG_ERR, "%s: cannot set effective gid by setregid(): %s\n", logstr, strerror(errno));
             }
@@ -674,7 +715,26 @@ static int plugin_run_or_verify(
 
 
     /* Set the new secondary group IDs */
-    if (setgroups(cntSecGid, secGid)!=0)
+    /* Note getgroups uses int for size (both as first argument and return
+     * value) */
+    /* Don't call setgroups with zero groups, at least not on MacOS/BSD: use
+     * primary group as secondary group in that case */
+    if (cntSecGid<0)	{
+	lcmaps_log(LOG_ERR, "%s: setgroups() called with negative number of groups.\n", logstr);
+        goto fail_posix;
+    }
+#ifdef __linux__
+    if (cntSecGid==0)
+	rc_sg=setgroups((size_t)cntSecGid, priGid);
+    else	
+	rc_sg=setgroups((size_t)cntSecGid, secGid);
+#else
+    if (cntSecGid==0)
+	rc_sg=setgroups(cntSecGid, priGid);
+    else
+	rc_sg=setgroups(cntSecGid, secGid);
+#endif
+    if (rc_sg != 0)
     {
         switch (errno)
         {
@@ -682,13 +742,11 @@ static int plugin_run_or_verify(
                 {
                     lcmaps_log_debug(LOG_NOTICE, "%s: Not that fatal but serious error: %s\n", logstr, strerror(errno));
                     goto fail_posix;
-                    break;
                 }
             case EPERM :
                 {
                     lcmaps_log(LOG_ERR, "%s: You are not ROOT: %s\n", logstr, strerror(errno));
                     goto fail_posix;
-                    break;
                 }
             case EINVAL :
                 {
@@ -699,7 +757,6 @@ static int plugin_run_or_verify(
                     lcmaps_log_debug(1, "%s: Invalid GID list\n", logstr);
                     lcmaps_log(LOG_ERR, "%s: %s\n", logstr, strerror(errno));
                     goto fail_posix;
-                    break;
                 }
             default :
                 {
@@ -713,7 +770,10 @@ static int plugin_run_or_verify(
     if (cntUid > 0)
     {
         if (set_only_euid)
-            if (setreuid(-1, uid[0]) != 0)
+	    /* Note: setreuid() -- both POSIX and Linux -- accepts -1 for the
+	     * first or second argument, even though a uid_t is an unsigned type
+	     * on Linux systems. */
+            if (setreuid((uid_t)-1, uid[0]) != 0)
             {
                 lcmaps_log(LOG_ERR, "%s: cannot set effective uid by setreuid(): %s\n", logstr, strerror(errno));
             }
@@ -775,7 +835,7 @@ static int plugin_run_or_verify(
 
 
     /* succes */
- success_posix:
+/* success_posix:*/
     if (list) free(list);
     lcmaps_log(LOG_INFO,"%s: posix_enf plugin succeeded\n", logstr);
     return LCMAPS_MOD_SUCCESS;
@@ -798,7 +858,7 @@ Returns:
 ******************************************************************************/
 int plugin_terminate()
 {
-    char * logstr = "lcmaps_plugin_posix_enf-plugin_introspect()";
+    const char * logstr = "lcmaps_plugin_posix_enf-plugin_introspect()";
 
     lcmaps_log_debug(4,"%s: terminating\n", logstr);
 
